@@ -529,6 +529,7 @@ avatar_chooser_maybe_convert_and_scale (EmpathyAvatarChooser *chooser,
 						   &converted_image_size,
 						   new_format_name,
 						   &error, NULL);
+		g_object_unref (pixbuf_scaled);
 
 		if (!saved) {
 			g_free (new_format_name);
@@ -578,13 +579,11 @@ avatar_chooser_clear_image (EmpathyAvatarChooser *chooser)
 	EmpathyAvatarChooserPriv *priv = GET_PRIV (chooser);
 	GtkWidget *image;
 
-	if (priv->avatar == NULL) {
-		return;
+	if (priv->avatar != NULL) {
+		empathy_avatar_unref (priv->avatar);
+		priv->avatar = NULL;
 	}
 
-	empathy_avatar_unref (priv->avatar);
-	priv->avatar = NULL;
-
 	image = gtk_image_new_from_icon_name ("stock_person", GTK_ICON_SIZE_DIALOG);
 	gtk_button_set_image (GTK_BUTTON (chooser), image);
 	g_signal_emit (chooser, signals[CHANGED], 0);
@@ -886,6 +885,8 @@ avatar_chooser_update_preview_cb (GtkFileChooser       *file_chooser,
 						  "gtk-dialog-question",
 						  GTK_ICON_SIZE_DIALOG);
 		}
+
+		g_free (filename);
 	}
 
 	gtk_file_chooser_set_preview_widget_active (file_chooser, TRUE);
