@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2007 Imendio AB
+ * Copyright (C) 2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  * 
- * Authors: Mikael Hallendal <micke@imendio.com>
- *          Richard Hult <richard@imendio.com>
- *          Martyn Russell <martyn@imendio.com>
+ * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
 #ifndef __EMPATHY_CHAT_VIEW_H__
@@ -34,41 +32,49 @@ G_BEGIN_DECLS
 
 #define EMPATHY_TYPE_CHAT_VIEW         (empathy_chat_view_get_type ())
 #define EMPATHY_CHAT_VIEW(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), EMPATHY_TYPE_CHAT_VIEW, EmpathyChatView))
-#define EMPATHY_CHAT_VIEW_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), EMPATHY_TYPE_CHAT_VIEW, EmpathyChatViewClass))
 #define EMPATHY_IS_CHAT_VIEW(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), EMPATHY_TYPE_CHAT_VIEW))
-#define EMPATHY_IS_CHAT_VIEW_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), EMPATHY_TYPE_CHAT_VIEW))
-#define EMPATHY_CHAT_VIEW_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), EMPATHY_TYPE_CHAT_VIEW, EmpathyChatViewClass))
+#define EMPATHY_TYPE_CHAT_VIEW_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), EMPATHY_TYPE_CHAT_VIEW, EmpathyChatViewIface))
 
 typedef struct _EmpathyChatView      EmpathyChatView;
-typedef struct _EmpathyChatViewClass EmpathyChatViewClass;
+typedef struct _EmpathyChatViewIface EmpathyChatViewIface;
 
-#include "empathy-theme.h"
+struct _EmpathyChatViewIface {
+	GTypeInterface   base_iface;
 
-struct _EmpathyChatView {
-	GtkTextView parent;
-	gpointer priv;
-};
-
-struct _EmpathyChatViewClass {
-	GtkTextViewClass parent_class;
+	/* VTabled */
+	void             (*append_message)       (EmpathyChatView *view,
+						  EmpathyMessage  *msg);
+	void             (*append_event)         (EmpathyChatView *view,
+						  const gchar     *str);
+	void             (*scroll)               (EmpathyChatView *view,
+						  gboolean         allow_scrolling);
+	void             (*scroll_down)          (EmpathyChatView *view);
+	gboolean         (*get_has_selection)    (EmpathyChatView *view);
+	void             (*clear)                (EmpathyChatView *view);
+	gboolean         (*find_previous)        (EmpathyChatView *view,
+						  const gchar     *search_criteria,
+						  gboolean         new_search);
+	gboolean         (*find_next)            (EmpathyChatView *view,
+						  const gchar     *search_criteria,
+						  gboolean         new_search);
+	void             (*find_abilities)       (EmpathyChatView *view,
+						  const gchar     *search_criteria,
+						  gboolean        *can_do_previous,
+						  gboolean        *can_do_next);
+	void             (*highlight)            (EmpathyChatView *view,
+						  const gchar     *text);
+	void             (*copy_clipboard)       (EmpathyChatView *view);
 };
 
 GType            empathy_chat_view_get_type             (void) G_GNUC_CONST;
-EmpathyChatView *empathy_chat_view_new                  (void);
 void             empathy_chat_view_append_message       (EmpathyChatView *view,
 							 EmpathyMessage  *msg);
 void             empathy_chat_view_append_event         (EmpathyChatView *view,
 							 const gchar     *str);
-void             empathy_chat_view_append_button        (EmpathyChatView *view,
-							 const gchar     *message,
-							 GtkWidget       *button1,
-							 GtkWidget       *button2);
 void             empathy_chat_view_scroll               (EmpathyChatView *view,
 							 gboolean         allow_scrolling);
 void             empathy_chat_view_scroll_down          (EmpathyChatView *view);
-gboolean         empathy_chat_view_get_selection_bounds (EmpathyChatView *view,
-							 GtkTextIter     *start,
-							 GtkTextIter     *end);
+gboolean         empathy_chat_view_get_has_selection    (EmpathyChatView *view);
 void             empathy_chat_view_clear                (EmpathyChatView *view);
 gboolean         empathy_chat_view_find_previous        (EmpathyChatView *view,
 							 const gchar     *search_criteria,
@@ -83,19 +89,8 @@ void             empathy_chat_view_find_abilities       (EmpathyChatView *view,
 void             empathy_chat_view_highlight            (EmpathyChatView *view,
 							 const gchar     *text);
 void             empathy_chat_view_copy_clipboard       (EmpathyChatView *view);
-EmpathyTheme *   empathy_chat_view_get_theme            (EmpathyChatView *view);
-void             empathy_chat_view_set_theme            (EmpathyChatView *view,
-							 EmpathyTheme    *theme);
-void             empathy_chat_view_set_margin           (EmpathyChatView *view,
-							 gint             margin);
-GtkWidget *      empathy_chat_view_get_smiley_menu      (GCallback        callback,
-							 gpointer         user_data);
-time_t           empathy_chat_view_get_last_timestamp   (EmpathyChatView *view);
-void             empathy_chat_view_set_last_timestamp   (EmpathyChatView *view,
-							 time_t           timestamp);
-EmpathyContact * empathy_chat_view_get_last_contact     (EmpathyChatView *view);
-GdkPixbuf *      empathy_chat_view_get_avatar_pixbuf_with_cache (EmpathyContact *contact);
 
 G_END_DECLS
 
 #endif /* __EMPATHY_CHAT_VIEW_H__ */
+
