@@ -111,6 +111,14 @@ empathy_contact_menu_new (EmpathyContact             *contact,
 	return menu;
 }
 
+static void
+empathy_contact_chat_menu_item_activated (GtkMenuItem *item,
+	EmpathyContact *contact)
+{
+  empathy_dispatcher_chat_with_contact (contact, NULL, NULL);
+}
+
+
 GtkWidget *
 empathy_contact_chat_menu_item_new (EmpathyContact *contact)
 {
@@ -125,8 +133,8 @@ empathy_contact_chat_menu_item_new (EmpathyContact *contact)
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	gtk_widget_show (image);
 
-	g_signal_connect_swapped (item, "activate",
-				  G_CALLBACK (empathy_dispatcher_chat_with_contact),
+	g_signal_connect (item, "activate",
+				  G_CALLBACK (empathy_contact_chat_menu_item_activated),
 				  contact);
 	
 	return item;
@@ -352,9 +360,7 @@ empathy_contact_invite_menu_item_new (EmpathyContact *contact)
 	GtkWidget *room_item;
 	EmpathyChatroomManager *mgr;
 	GList *rooms, *l;
-	GtkWidget *submenu;
-	GtkMenuShell *submenu_shell;
-	gboolean have_rooms = FALSE;
+	GtkWidget *submenu = NULL;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
@@ -367,27 +373,23 @@ empathy_contact_invite_menu_item_new (EmpathyContact *contact)
 	rooms = empathy_chatroom_manager_get_chatrooms (mgr,
 		empathy_contact_get_account (contact));
 
-	/* create rooms sub menu */
-	submenu = gtk_menu_new ();
-	submenu_shell = GTK_MENU_SHELL (submenu);
-
 	for (l = rooms; l != NULL; l = g_list_next (l)) {
 		EmpathyChatroom *chatroom = l->data;
 
 		if (empathy_chatroom_get_tp_chat (chatroom) != NULL) {
-			have_rooms = TRUE;
+			if (G_UNLIKELY (submenu == NULL))
+				submenu = gtk_menu_new ();
 
 			room_item = create_room_sub_menu (contact, chatroom);
-			gtk_menu_shell_append (submenu_shell, room_item);
+			gtk_menu_shell_append ((GtkMenuShell*)submenu, room_item);
 			gtk_widget_show (room_item);
 		}
 	}
 
-	if (have_rooms) {
+	if (submenu) {
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
 	} else {
 		gtk_widget_set_sensitive (item, FALSE);
-		gtk_widget_destroy (submenu);
 	}
 
 	gtk_widget_show (image);
