@@ -44,6 +44,8 @@ struct _SmileyManagerTree {
 
 G_DEFINE_TYPE (EmpathySmileyManager, empathy_smiley_manager, G_TYPE_OBJECT);
 
+static EmpathySmileyManager *manager_singleton = NULL;
+
 static SmileyManagerTree *
 smiley_manager_tree_new (gunichar c)
 {
@@ -77,8 +79,9 @@ smiley_manager_tree_free (SmileyManagerTree *tree)
 	g_slice_free (SmileyManagerTree, tree);
 }
 
+/* Note: This function takes the ownership of str */
 static EmpathySmiley *
-smiley_new (GdkPixbuf *pixbuf, const gchar *str)
+smiley_new (GdkPixbuf *pixbuf, gchar *str)
 {
 	EmpathySmiley *smiley;
 
@@ -86,7 +89,7 @@ smiley_new (GdkPixbuf *pixbuf, const gchar *str)
 	if (pixbuf) {
 		smiley->pixbuf = g_object_ref (pixbuf);
 	}
-	smiley->str = g_strdup (str);
+	smiley->str = str;
 
 	return smiley;
 }
@@ -115,12 +118,33 @@ smiley_manager_finalize (GObject *object)
 	g_slist_free (priv->smileys);
 }
 
+static GObject *
+smiley_manager_constructor (GType type,
+			    guint n_props,
+			    GObjectConstructParam *props)
+{
+	GObject *retval;
+
+	if (manager_singleton) {
+		retval = g_object_ref (manager_singleton);
+	} else {
+		retval = G_OBJECT_CLASS (empathy_smiley_manager_parent_class)->constructor
+			(type, n_props, props);
+
+		manager_singleton = EMPATHY_SMILEY_MANAGER (retval);
+		g_object_add_weak_pointer (retval, (gpointer) &manager_singleton);
+	}
+
+	return retval;
+}
+
 static void
 empathy_smiley_manager_class_init (EmpathySmileyManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = smiley_manager_finalize;
+	object_class->constructor = smiley_manager_constructor;
 
 	g_type_class_add_private (object_class, sizeof (EmpathySmileyManagerPriv));
 }
@@ -134,22 +158,14 @@ empathy_smiley_manager_init (EmpathySmileyManager *manager)
 	manager->priv = priv;
 	priv->tree = smiley_manager_tree_new ('\0');
 	priv->smileys = NULL;
+
+	empathy_smiley_manager_load (manager);
 }
 
 EmpathySmileyManager *
-empathy_smiley_manager_new (void)
+empathy_smiley_manager_dup_singleton (void)
 {
-	static EmpathySmileyManager *manager = NULL;
-
-	if (!manager) {
-		manager = g_object_new (EMPATHY_TYPE_SMILEY_MANAGER, NULL);
-		g_object_add_weak_pointer (G_OBJECT (manager), (gpointer) &manager);
-		empathy_smiley_manager_load (manager);
-	} else {
-		g_object_ref (manager);
-	}
-
-	return manager;
+	return g_object_new (EMPATHY_TYPE_SMILEY_MANAGER, NULL);
 }
 
 static SmileyManagerTree *
@@ -214,7 +230,8 @@ smiley_manager_add_valist (EmpathySmileyManager *manager,
 		smiley_manager_tree_insert (priv->tree, smiley, str);
 	}
 
-	priv->smileys = g_slist_prepend (priv->smileys, smiley_new (smiley, first_str));
+	priv->smileys = g_slist_prepend (priv->smileys,
+		smiley_new (smiley, g_strdup (first_str)));
 }
 
 void
@@ -227,8 +244,8 @@ empathy_smiley_manager_add (EmpathySmileyManager *manager,
 	va_list    var_args;
 
 	g_return_if_fail (EMPATHY_IS_SMILEY_MANAGER (manager));
-	g_return_if_fail (!G_STR_EMPTY (icon_name));
-	g_return_if_fail (!G_STR_EMPTY (first_str));
+	g_return_if_fail (!EMP_STR_EMPTY (icon_name));
+	g_return_if_fail (!EMP_STR_EMPTY (first_str));
 
 	smiley = empathy_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	if (smiley) {
@@ -249,7 +266,7 @@ empathy_smiley_manager_add_from_pixbuf (EmpathySmileyManager *manager,
 
 	g_return_if_fail (EMPATHY_IS_SMILEY_MANAGER (manager));
 	g_return_if_fail (GDK_IS_PIXBUF (smiley));
-	g_return_if_fail (!G_STR_EMPTY (first_str));
+	g_return_if_fail (!EMP_STR_EMPTY (first_str));
 
 	va_start (var_args, first_str);
 	smiley_manager_add_valist (manager, smiley, first_str, var_args);
