@@ -38,16 +38,28 @@
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <glade/glade.h>
+#include <canberra-gtk.h>
 
 #include <libmissioncontrol/mc-profile.h>
 
 #include "empathy-ui-utils.h"
 #include "empathy-images.h"
+#include "empathy-conf.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-idle.h>
+#include <libempathy/empathy-tp-file.h>
+
+#define SCHEMES "(https?|s?ftps?|nntp|news|javascript|about|ghelp|apt|telnet|"\
+		"file|webcal|mailto)"
+#define BODY "([^\\ \\n]+)"
+#define END_BODY "([^\\ \\n]*[^,;\?><()\\ \"\\.\\n])"
+#define URI_REGEX "("SCHEMES"://"END_BODY")" \
+		  "|((mailto:)?"BODY"@"BODY"\\."END_BODY")"\
+		  "|((www|ftp)\\."END_BODY")"
 
 void
 empathy_gtk_init (void)
@@ -64,6 +76,19 @@ empathy_gtk_init (void)
 	initialized = TRUE;
 }
 
+GRegex *
+empathy_uri_regex_dup_singleton (void)
+{
+	static GRegex *uri_regex = NULL;
+
+	/* We intentionally leak the regex so it's not recomputed */
+	if (!uri_regex) {
+		uri_regex = g_regex_new (URI_REGEX, 0, 0, NULL);
+	}
+
+	return g_regex_ref (uri_regex);
+}
+
 struct SizeData {
 	gint     width;
 	gint     height;
@@ -540,14 +565,12 @@ empathy_pixbuf_scale_down_if_necessary (GdkPixbuf *pixbuf, gint max_size)
 }
 
 GdkPixbuf *
-empathy_pixbuf_from_icon_name (const gchar *icon_name,
-			      GtkIconSize  icon_size)
+empathy_pixbuf_from_icon_name_sized (const gchar *icon_name,
+				     gint size)
 {
-	GtkIconTheme  *theme;
-	GdkPixbuf     *pixbuf = NULL;
-	GError        *error = NULL;
-	gint           w, h;
-	gint           size = 48;
+	GtkIconTheme *theme;
+	GdkPixbuf *pixbuf;
+	GError *error = NULL;
 
 	if (!icon_name) {
 		return NULL;
@@ -555,10 +578,6 @@ empathy_pixbuf_from_icon_name (const gchar *icon_name,
 
 	theme = gtk_icon_theme_get_default ();
 
-	if (gtk_icon_size_lookup (icon_size, &w, &h)) {
-		size = (w + h) / 2;
-	}
-
 	pixbuf = gtk_icon_theme_load_icon (theme,
 					   icon_name,
 					   size,
@@ -572,6 +591,24 @@ empathy_pixbuf_from_icon_name (const gchar *icon_name,
 	return pixbuf;
 }
 
+GdkPixbuf *
+empathy_pixbuf_from_icon_name (const gchar *icon_name,
+			       GtkIconSize  icon_size)
+{
+	gint  w, h;
+	gint  size = 48;
+
+	if (!icon_name) {
+		return NULL;
+	}
+
+	if (gtk_icon_size_lookup (icon_size, &w, &h)) {
+		size = (w + h) / 2;
+	}
+
+	return empathy_pixbuf_from_icon_name_sized (icon_name, size);
+}
+
 /* Stolen from GtkSourceView, hence the weird intendation. Please keep it like
  * that to make it easier to apply changes from the original code.
  */
@@ -1418,6 +1455,29 @@ empathy_toggle_button_set_state_quietly (GtkWidget *widget,
 
 /* Sending files with the file chooser */
 
+static void
+file_manager_send_file_request_cb (EmpathyDispatchOperation *operation,
+				   const GError *error, gpointer user_data)
+{
+	GFile *file = (GFile *)user_data;
+	EmpathyTpFile *tp_file;
+
+	if (error != NULL) {
+		DEBUG ("Couldn't request channel: %s", error->message);
+		g_object_unref (file);
+		return;
+	}
+
+	DEBUG ("Starting to send file");
+
+	tp_file = EMPATHY_TP_FILE (
+		empathy_dispatch_operation_get_channel_wrapper (operation));
+
+	empathy_tp_file_offer (tp_file, file, NULL);
+
+	g_object_unref (file);
+}
+
 static void
 file_manager_send_file_response_cb (GtkDialog      *widget,
 				    gint            response_id,
@@ -1434,18 +1494,42 @@ file_manager_send_file_response_cb (GtkDialog      *widget,
 		for (l = list; l; l = l->next) {
 			gchar            *uri;
 			GFile            *gfile;
+			GFileInfo        *info;
 			GtkRecentManager *manager;
+			gchar *filename;
+			GTimeVal mtime;
+			GError *error = NULL;
 
 			uri = l->data;
 			gfile = g_file_new_for_uri (uri);
+			info = g_file_query_info (gfile,
+				G_FILE_ATTRIBUTE_STANDARD_SIZE ","
+				G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+				G_FILE_ATTRIBUTE_TIME_MODIFIED,
+				0, NULL, &error);
+
+			if (error) {
+				DEBUG ("Can't get info about the file: %s", error->message);
+				g_clear_error (&error);
+				g_object_unref (gfile);
+				continue;
+			}
 
 			DEBUG ("\t%s", uri);
-			empathy_dispatcher_send_file (contact, gfile);
+			filename = g_file_get_basename (gfile);
+			g_file_info_get_modification_time (info, &mtime);
+
+			empathy_dispatcher_send_file_to_contact (contact,
+				filename, g_file_info_get_size (info), mtime.tv_sec,
+				g_file_info_get_content_type (info),
+				file_manager_send_file_request_cb, gfile);
+
+			g_free (filename);
+			g_object_unref (info);
 
 			manager = gtk_recent_manager_get_default ();
 			gtk_recent_manager_add_item (manager, uri);
 
-			g_object_unref (gfile);
 			g_free (uri);
 		}
 		g_slist_free (list);
@@ -1490,3 +1574,138 @@ empathy_send_file_with_file_chooser (EmpathyContact *contact)
 	gtk_widget_show (widget);
 }
 
+typedef struct {
+	EmpathySound sound_id;
+	const char * event_ca_id;
+	const char * event_ca_description;
+	const char * gconf_key;
+} EmpathySoundEntry;
+
+/* NOTE: these entries MUST be in the same order than EmpathySound enum */
+static EmpathySoundEntry sound_entries[LAST_EMPATHY_SOUND] = {
+	{ EMPATHY_SOUND_MESSAGE_INCOMING, "message-new-instant",
+	  N_("Received an instant message"), EMPATHY_PREFS_SOUNDS_INCOMING_MESSAGE } ,
+	{ EMPATHY_SOUND_MESSAGE_OUTGOING, "message-sent-instant",
+	  N_("Sent an instant message"), EMPATHY_PREFS_SOUNDS_OUTGOING_MESSAGE } ,
+	{ EMPATHY_SOUND_CONVERSATION_NEW, "message-new-instant",
+	  N_("Incoming chat request"), EMPATHY_PREFS_SOUNDS_NEW_CONVERSATION },
+	{ EMPATHY_SOUND_CONTACT_CONNECTED, "service-login",
+	  N_("Contact connected"), EMPATHY_PREFS_SOUNDS_CONTACT_LOGIN },
+	{ EMPATHY_SOUND_CONTACT_DISCONNECTED, "service-logout",
+	  N_("Contact disconnected"), EMPATHY_PREFS_SOUNDS_CONTACT_LOGOUT },
+	{ EMPATHY_SOUND_ACCOUNT_CONNECTED, "service-login",
+	  N_("Connected to server"), EMPATHY_PREFS_SOUNDS_SERVICE_LOGIN },
+	{ EMPATHY_SOUND_ACCOUNT_DISCONNECTED, "service-logout",
+	  N_("Disconnected from server"), EMPATHY_PREFS_SOUNDS_SERVICE_LOGOUT },
+	{ EMPATHY_SOUND_PHONE_INCOMING, "phone-incoming-call",
+	  N_("Incoming voice call"), NULL },
+	{ EMPATHY_SOUND_PHONE_OUTGOING, "phone-outgoing-calling",
+	  N_("Outgoing voice call"), NULL },
+	{ EMPATHY_SOUND_PHONE_HANGUP, "phone-hangup",
+	  N_("Voice call ended"), NULL },
+};
+
+
+static gboolean
+empathy_sound_pref_is_enabled (const char *key)
+{
+	EmpathyConf *conf;
+	gboolean res;
+
+	conf = empathy_conf_get ();
+	res = FALSE;
+
+	empathy_conf_get_bool (conf, EMPATHY_PREFS_SOUNDS_ENABLED, &res);
+
+	if (!res) {
+		return FALSE;
+	}
+
+	if (!empathy_check_available_state ()) {
+		empathy_conf_get_bool (conf, EMPATHY_PREFS_SOUNDS_DISABLED_AWAY,
+				       &res);
+		if (res) {
+			return FALSE;
+		}
+	}
+
+	empathy_conf_get_bool (conf, key, &res);
+
+	return res;
+}
+
+void
+empathy_sound_stop (EmpathySound sound_id)
+{
+	EmpathySoundEntry *entry;
+
+	g_return_if_fail (sound_id < LAST_EMPATHY_SOUND);
+
+	entry = &(sound_entries[sound_id]);
+	g_return_if_fail (entry->sound_id == sound_id);
+
+	ca_context_cancel (ca_gtk_context_get (), entry->sound_id);
+}
+
+
+gboolean
+empathy_sound_play_full (GtkWidget *widget, EmpathySound sound_id,
+	ca_finish_callback_t callback, gpointer user_data)
+{
+	EmpathySoundEntry *entry;
+	gboolean should_play = TRUE;
+	ca_proplist *p = NULL;
+	ca_context *c;
+
+	g_return_val_if_fail (sound_id < LAST_EMPATHY_SOUND, FALSE);
+
+	entry = &(sound_entries[sound_id]);
+	g_return_val_if_fail (entry->sound_id == sound_id, FALSE);
+
+	if (entry->gconf_key != NULL) {
+		should_play = empathy_sound_pref_is_enabled (entry->gconf_key);
+	}
+
+	if (!should_play)
+		return FALSE;
+
+	c = ca_gtk_context_get ();
+	ca_context_cancel (c, entry->sound_id);
+
+	DEBUG ("Play sound \"%s\" (%s)",
+	       entry->event_ca_id,
+	       entry->event_ca_description);
+
+	if (ca_proplist_create(&p) < 0)
+		goto failed;
+
+	if (ca_proplist_sets (p, CA_PROP_EVENT_ID, entry->event_ca_id) < 0)
+		goto failed;
+
+	if (ca_proplist_sets (p, CA_PROP_EVENT_DESCRIPTION,
+			gettext (entry->event_ca_id)) < 0)
+		goto failed;
+
+	if (ca_gtk_proplist_set_for_widget (p, widget) < 0)
+		goto failed;
+
+	ca_context_play_full (ca_gtk_context_get (), entry->sound_id,
+		p, callback, user_data);
+
+	ca_proplist_destroy (p);
+
+	return TRUE;
+
+failed:
+	if (p != NULL)
+		ca_proplist_destroy (p);
+
+	return FALSE;
+}
+
+void
+empathy_sound_play (GtkWidget *widget, EmpathySound sound_id)
+{
+	empathy_sound_play_full (widget, sound_id, NULL, NULL);
+}
+
