@@ -162,9 +162,8 @@ account_deleted_cb (McAccountMonitor *mon,
 
   if (account)
     {
-      g_signal_emit (manager, signals[ACCOUNT_DELETED], 0, account);
-
       g_hash_table_remove (priv->accounts, account);
+      g_signal_emit (manager, signals[ACCOUNT_DELETED], 0, account);
       g_object_unref (account);
     }
 }
@@ -369,13 +368,12 @@ empathy_account_manager_init (EmpathyAccountManager *manager)
 {
   EmpathyAccountManagerPriv *priv;
   GList *mc_accounts, *l;
-  AccountData *data;
 
   priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
 				      EMPATHY_TYPE_ACCOUNT_MANAGER, EmpathyAccountManagerPriv);
   manager->priv = priv;
   priv->monitor = mc_account_monitor_new ();
-  priv->mc = empathy_mission_control_new ();
+  priv->mc = empathy_mission_control_dup_singleton ();
   priv->connected = priv->connecting = 0;
   priv->dispose_run = FALSE;
 
@@ -387,11 +385,7 @@ empathy_account_manager_init (EmpathyAccountManager *manager)
   mc_accounts = mc_accounts_list ();
 
   for (l = mc_accounts; l; l = l->next)
-    {
-      data = account_data_new_default (priv->mc, l->data);
-
-      account_created_cb (priv->monitor, (char *) mc_account_get_unique_name (l->data), manager);
-    }
+    account_created_cb (priv->monitor, (char *) mc_account_get_unique_name (l->data), manager);
 
   g_signal_connect (priv->monitor, "account-created",
                     G_CALLBACK (account_created_cb), manager);
@@ -475,8 +469,8 @@ do_constructor (GType type,
                                                                                    n_construct_params,
                                                                                    construct_params);
       manager_singleton = EMPATHY_ACCOUNT_MANAGER (retval);
-      g_object_add_weak_pointer (retval, (gpointer *) &manager_singleton);
-    } 
+      g_object_add_weak_pointer (retval, (gpointer) &manager_singleton);
+    }
   else
     {
       retval = g_object_ref (manager_singleton);
@@ -621,3 +615,22 @@ empathy_account_manager_is_account_just_connected (EmpathyAccountManager *manage
   return (data->source_id > 0);
 }
 
+/**
+ * empathy_account_manager_get_count:
+ * @manager: a #EmpathyAccountManager
+ *
+ * Get the number of accounts.
+ *
+ * Returns: the number of accounts.
+ **/
+int
+empathy_account_manager_get_count (EmpathyAccountManager *manager)
+{
+  EmpathyAccountManagerPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_ACCOUNT_MANAGER (manager), 0);
+
+  priv = GET_PRIV (manager);
+
+  return g_hash_table_size (priv->accounts);
+}
