@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
 /*
  * Copyright (C) 2004 Imendio AB
  * Copyright (C) 2007-2008 Collabora Ltd.
@@ -21,25 +21,30 @@
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
+ *          Sjoerd Simons <sjoerd.simons@collabora.co.uk>
  */
 
 #include "config.h"
 
 #include <string.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <telepathy-glib/util.h>
+#include <libmissioncontrol/mc-enum-types.h>
 
 #include "empathy-contact.h"
+#include "empathy-contact-factory.h"
 #include "empathy-utils.h"
 #include "empathy-enum-types.h"
+#include "empathy-marshal.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include "empathy-debug.h"
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyContact)
 typedef struct {
+  EmpathyContactFactory *factory;
   gchar *id;
   gchar *name;
   EmpathyAvatar *avatar;
@@ -51,8 +56,17 @@ typedef struct {
   gboolean is_user;
   guint hash;
   EmpathyContactReady ready;
+  GList *ready_callbacks;
 } EmpathyContactPriv;
 
+typedef struct {
+    EmpathyContactReady ready;
+    EmpathyContactReadyCb *callback;
+    gpointer user_data;
+    GDestroyNotify destroy;
+    GObject *weak_object;
+} ReadyCbData;
+
 static void contact_finalize (GObject *object);
 static void contact_get_property (GObject *object, guint param_id,
     GValue *value, GParamSpec *pspec);
@@ -76,6 +90,29 @@ enum
   PROP_READY
 };
 
+enum {
+  PRESENCE_CHANGED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
+static void
+contact_dispose (GObject *object)
+{
+  EmpathyContactPriv *priv = GET_PRIV (object);
+
+  if (priv->account)
+    g_object_unref (priv->account);
+  priv->account = NULL;
+
+  if (priv->factory)
+    g_object_unref (priv->factory);
+  priv->factory = NULL;
+
+  G_OBJECT_CLASS (empathy_contact_parent_class)->dispose (object);
+}
+
 static void
 empathy_contact_class_init (EmpathyContactClass *class)
 {
@@ -84,6 +121,7 @@ empathy_contact_class_init (EmpathyContactClass *class)
   object_class = G_OBJECT_CLASS (class);
 
   object_class->finalize = contact_finalize;
+  object_class->dispose = contact_dispose;
   object_class->get_property = contact_get_property;
   object_class->set_property = contact_set_property;
 
@@ -173,6 +211,17 @@ empathy_contact_class_init (EmpathyContactClass *class)
         EMPATHY_CONTACT_READY_NONE,
         G_PARAM_READABLE));
 
+  signals[PRESENCE_CHANGED] =
+    g_signal_new ("presence-changed",
+                  G_TYPE_FROM_CLASS (class),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  _empathy_marshal_VOID__ENUM_ENUM,
+                  G_TYPE_NONE,
+                  2, MC_TYPE_PRESENCE,
+                  MC_TYPE_PRESENCE);
+
   g_type_class_add_private (object_class, sizeof (EmpathyContactPriv));
 }
 
@@ -183,12 +232,17 @@ empathy_contact_init (EmpathyContact *contact)
     EMPATHY_TYPE_CONTACT, EmpathyContactPriv);
 
   contact->priv = priv;
+
+  /* Keep a ref to the factory to be sure it is not finalized while there is
+   * still contacts alive. */
+  priv->factory = empathy_contact_factory_dup_singleton ();
 }
 
 static void
 contact_finalize (GObject *object)
 {
   EmpathyContactPriv *priv;
+  GList *l;
 
   priv = GET_PRIV (object);
 
@@ -198,12 +252,21 @@ contact_finalize (GObject *object)
   g_free (priv->id);
   g_free (priv->presence_message);
 
+  for (l = priv->ready_callbacks; l != NULL; l = g_list_next (l))
+    {
+      ReadyCbData *d = (ReadyCbData *)l->data;
+
+      if (d->destroy != NULL)
+        d->destroy (d->user_data);
+      g_slice_free (ReadyCbData, d);
+    }
+
+  g_list_free (priv->ready_callbacks);
+  priv->ready_callbacks = NULL;
+
   if (priv->avatar)
       empathy_avatar_unref (priv->avatar);
 
-  if (priv->account)
-      g_object_unref (priv->account);
-
   G_OBJECT_CLASS (empathy_contact_parent_class)->finalize (object);
 }
 
@@ -310,6 +373,60 @@ contact_set_property (GObject *object,
     };
 }
 
+static gboolean
+contact_is_ready (EmpathyContact *contact, EmpathyContactReady ready)
+{
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+
+  /* When the name is NULL, empathy_contact_get_name() fallback to the id.
+   * When the caller want to wait the name to be ready, it also want to wait
+   * the id to be ready in case of fallback. */
+  if ((ready & EMPATHY_CONTACT_READY_NAME) && EMP_STR_EMPTY (priv->name))
+      ready |= EMPATHY_CONTACT_READY_ID;
+
+  return (priv->ready & ready) == ready;
+}
+
+static void
+contact_weak_object_notify (gpointer data, GObject *old_object)
+{
+  EmpathyContact *contact = EMPATHY_CONTACT (data);
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+
+  GList *l, *ln;
+
+  for (l = priv->ready_callbacks ; l != NULL ; l = ln )
+    {
+      ReadyCbData *d = (ReadyCbData *)l->data;
+      ln = g_list_next (l);
+
+      if (d->weak_object == old_object)
+        {
+          if (d->destroy != NULL)
+            d->destroy (d->user_data);
+
+          priv->ready_callbacks = g_list_delete_link (priv->ready_callbacks,
+            l);
+
+          g_slice_free (ReadyCbData, d);
+        }
+    }
+}
+
+static void
+contact_call_ready_callback (EmpathyContact *contact, const GError *error,
+  ReadyCbData *data)
+{
+  data->callback (contact, error, data->user_data, data->weak_object);
+  if (data->destroy != NULL)
+    data->destroy (data->user_data);
+
+  if (data->weak_object)
+    g_object_weak_unref (data->weak_object,
+      contact_weak_object_notify, contact);
+}
+
+
 static void
 contact_set_ready_flag (EmpathyContact *contact,
                         EmpathyContactReady flag)
@@ -318,8 +435,37 @@ contact_set_ready_flag (EmpathyContact *contact,
 
   if (!(priv->ready & flag))
     {
+      GList *l, *ln;
+
       priv->ready |= flag;
       g_object_notify (G_OBJECT (contact), "ready");
+
+      for (l = priv->ready_callbacks ; l != NULL ; l = ln )
+        {
+          ReadyCbData *d = (ReadyCbData *)l->data;
+          ln = g_list_next (l);
+
+          if (contact_is_ready (contact, d->ready))
+            {
+              contact_call_ready_callback (contact, NULL, d);
+              priv->ready_callbacks = g_list_delete_link
+                (priv->ready_callbacks, l);
+              g_slice_free (ReadyCbData, d);
+            }
+        }
+    }
+}
+
+static void
+contact_remove_ready_flag (EmpathyContact *contact,
+                           EmpathyContactReady flag)
+{
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+
+  if (priv->ready & flag)
+    {
+      priv->ready ^= flag;
+      g_object_notify (G_OBJECT (contact), "ready");
     }
 }
 
@@ -375,7 +521,7 @@ empathy_contact_set_id (EmpathyContact *contact,
       priv->id = g_strdup (id);
 
       g_object_notify (G_OBJECT (contact), "id");
-      if (G_STR_EMPTY (priv->name))
+      if (EMP_STR_EMPTY (priv->name))
           g_object_notify (G_OBJECT (contact), "name");
     }
   contact_set_ready_flag (contact, EMPATHY_CONTACT_READY_ID);
@@ -392,7 +538,7 @@ empathy_contact_get_name (EmpathyContact *contact)
 
   priv = GET_PRIV (contact);
 
-  if (G_STR_EMPTY (priv->name))
+  if (EMP_STR_EMPTY (priv->name))
       return empathy_contact_get_id (contact);
 
   return priv->name;
@@ -506,6 +652,7 @@ empathy_contact_set_presence (EmpathyContact *contact,
                               McPresence presence)
 {
   EmpathyContactPriv *priv;
+  McPresence old_presence;
 
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
@@ -514,8 +661,11 @@ empathy_contact_set_presence (EmpathyContact *contact,
   if (presence == priv->presence)
     return;
 
+  old_presence = priv->presence;
   priv->presence = presence;
 
+  g_signal_emit (contact, signals[PRESENCE_CHANGED], 0, presence, old_presence);
+
   g_object_notify (G_OBJECT (contact), "presence");
 }
 
@@ -576,7 +726,12 @@ empathy_contact_set_handle (EmpathyContact *contact,
       priv->handle = handle;
       g_object_notify (G_OBJECT (contact), "handle");
     }
-  contact_set_ready_flag (contact, EMPATHY_CONTACT_READY_HANDLE);
+
+  if (handle != 0)
+    contact_set_ready_flag (contact, EMPATHY_CONTACT_READY_HANDLE);
+  else
+    contact_remove_ready_flag (contact, EMPATHY_CONTACT_READY_HANDLE);
+
   g_object_unref (contact);
 }
 
@@ -744,22 +899,43 @@ empathy_contact_hash (gconstpointer key)
   return priv->hash;
 }
 
+void empathy_contact_call_when_ready (EmpathyContact *contact,
+  EmpathyContactReady ready, EmpathyContactReadyCb *callback,
+  gpointer user_data, GDestroyNotify destroy, GObject *weak_object)
+{
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+
+  g_return_if_fail (contact != NULL);
+  g_return_if_fail (callback != NULL);
+
+  if (contact_is_ready (contact, ready))
+    {
+      callback (contact, NULL, user_data, weak_object);
+      if (destroy != NULL)
+        destroy (user_data);
+    }
+  else
+    {
+      ReadyCbData *d = g_slice_new0 (ReadyCbData);
+      d->ready = ready;
+      d->callback = callback;
+      d->user_data = user_data;
+      d->destroy = destroy;
+      d->weak_object = weak_object;
+
+      if (weak_object != NULL)
+        g_object_weak_ref (weak_object, contact_weak_object_notify, contact);
+
+      priv->ready_callbacks = g_list_prepend (priv->ready_callbacks, d);
+    }
+}
+
 static gboolean
 contact_is_ready_func (GObject *contact,
                        gpointer user_data)
 {
-  EmpathyContactPriv *priv = GET_PRIV (contact);
-  EmpathyContactReady ready;
-
-  ready = GPOINTER_TO_UINT (user_data);
-
-  /* When the name is NULL, empathy_contact_get_name() fallback to the id.
-   * When the caller want to wait the name to be ready, it also want to wait
-   * the id to be ready in case of fallback. */
-  if ((ready & EMPATHY_CONTACT_READY_NAME) && G_STR_EMPTY (priv->name))
-      ready |= EMPATHY_CONTACT_READY_ID;
-
-  return (priv->ready & ready) == ready;
+  return contact_is_ready (EMPATHY_CONTACT (contact),
+    GPOINTER_TO_UINT (user_data));
 }
 
 void
@@ -782,7 +958,7 @@ contact_get_avatar_filename (EmpathyContact *contact,
   gchar *token_escaped;
   gchar *contact_escaped;
 
-  if (G_STR_EMPTY (priv->id))
+  if (EMP_STR_EMPTY (priv->id))
     return NULL;
 
   contact_escaped = tp_escape_as_identifier (priv->id);
@@ -820,7 +996,7 @@ empathy_contact_load_avatar_data (EmpathyContact *contact,
   g_return_if_fail (data != NULL);
   g_return_if_fail (len > 0);
   g_return_if_fail (format != NULL);
-  g_return_if_fail (!G_STR_EMPTY (token));
+  g_return_if_fail (!EMP_STR_EMPTY (token));
 
   /* Load and set the avatar */
   avatar = empathy_avatar_new (g_memdup (data, len), len, g_strdup (format),
@@ -855,7 +1031,7 @@ empathy_contact_load_avatar_cache (EmpathyContact *contact,
   GError *error = NULL;
 
   g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), FALSE);
-  g_return_val_if_fail (!G_STR_EMPTY (token), FALSE);
+  g_return_val_if_fail (!EMP_STR_EMPTY (token), FALSE);
 
   /* Load the avatar from file if it exists */
   filename = contact_get_avatar_filename (contact, token);
