@@ -830,7 +830,6 @@ dispatcher_connection_ready_cb (TpConnection *connection,
     }
 
   /* Advertise VoIP capabilities */
-  /* FIXME: Capabilities is leaked */
   capabilities = g_ptr_array_sized_new (1);
   cap_type = dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING,
     G_TYPE_UINT, G_TYPE_INVALID);
@@ -848,6 +847,9 @@ dispatcher_connection_ready_cb (TpConnection *connection,
     connection, -1, capabilities, &remove,
     dispatcher_connection_advertise_capabilities_cb,
     NULL, NULL, G_OBJECT (dispatcher));
+
+  g_value_unset (&cap);
+  g_ptr_array_free (capabilities, TRUE);
 }
 
 static void
