@@ -53,6 +53,7 @@ typedef struct {
 
 	McPresence      away_saved_state;
 	McPresence      nm_saved_state;
+	gchar          *nm_saved_status;
 
 	gboolean        is_idle;
 	gboolean        nm_connected;
@@ -76,7 +77,9 @@ enum {
 	PROP_USE_NM
 };
 
-G_DEFINE_TYPE (EmpathyIdle, empathy_idle, G_TYPE_OBJECT)
+G_DEFINE_TYPE (EmpathyIdle, empathy_idle, G_TYPE_OBJECT);
+
+static EmpathyIdle * idle_singleton = NULL;
 
 static void
 idle_presence_changed_cb (MissionControl *mc,
@@ -93,7 +96,7 @@ idle_presence_changed_cb (MissionControl *mc,
 	g_free (priv->status);
 	priv->state = state;
 	priv->status = NULL;
-	if (!G_STR_EMPTY (status)) {
+	if (!EMP_STR_EMPTY (status)) {
 		priv->status = g_strdup (status);
 	}
 
@@ -240,15 +243,23 @@ idle_nm_state_change_cb (DBusGProxy  *proxy,
 
 	if (old_nm_connected && !new_nm_connected) {
 		/* We are no more connected */
-		DEBUG ("Disconnected: Save state %d", priv->state);
+		DEBUG ("Disconnected: Save state %d (%s)",
+				priv->state, priv->status);
 		priv->nm_saved_state = priv->state;
+		g_free (priv->nm_saved_status);
+		priv->nm_saved_status = g_strdup (priv->status);
 		empathy_idle_set_state (idle, MC_PRESENCE_OFFLINE);
 	}
 	else if (!old_nm_connected && new_nm_connected) {
 		/* We are now connected */
-		DEBUG ("Reconnected: Restore state %d", priv->nm_saved_state);
-		empathy_idle_set_state (idle, priv->nm_saved_state);
+		DEBUG ("Reconnected: Restore state %d (%s)",
+				priv->nm_saved_state, priv->nm_saved_status);
+		empathy_idle_set_presence (idle,
+				priv->nm_saved_state,
+				priv->nm_saved_status);
 		priv->nm_saved_state = MC_PRESENCE_UNSET;
+		g_free (priv->nm_saved_status);
+		priv->nm_saved_status = NULL;
 	}
 
 	priv->nm_connected = new_nm_connected;
@@ -271,6 +282,26 @@ idle_finalize (GObject *object)
 	idle_ext_away_stop (EMPATHY_IDLE (object));
 }
 
+static GObject *
+idle_constructor (GType type,
+		  guint n_props,
+		  GObjectConstructParam *props)
+{
+	GObject *retval;
+
+	if (idle_singleton) {
+		retval = g_object_ref (idle_singleton);
+	} else {
+		retval = G_OBJECT_CLASS (empathy_idle_parent_class)->constructor
+			(type, n_props, props);
+
+		idle_singleton = EMPATHY_IDLE (retval);
+		g_object_add_weak_pointer (retval, (gpointer) &idle_singleton);
+	}
+
+	return retval;
+}
+
 static void
 idle_get_property (GObject    *object,
 		   guint       param_id,
@@ -345,6 +376,7 @@ empathy_idle_class_init (EmpathyIdleClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = idle_finalize;
+	object_class->constructor = idle_constructor;
 	object_class->get_property = idle_get_property;
 	object_class->set_property = idle_set_property;
 
@@ -401,12 +433,23 @@ empathy_idle_init (EmpathyIdle *idle)
 
 	idle->priv = priv;
 	priv->is_idle = FALSE;
-	priv->mc = empathy_mission_control_new ();
-	priv->state = mission_control_get_presence_actual (priv->mc, NULL);
-	priv->status = mission_control_get_presence_message_actual (priv->mc, NULL);
-	if (G_STR_EMPTY (priv->status)) {
+	priv->mc = empathy_mission_control_dup_singleton ();
+	priv->state = mission_control_get_presence_actual (priv->mc, &error);
+	if (error) {
+		DEBUG ("Error getting actual presence: %s", error->message);
+
+		priv->state = MC_PRESENCE_UNSET;
+		g_clear_error (&error);
+	}
+	priv->status = mission_control_get_presence_message_actual (priv->mc, &error);
+	if (error || EMP_STR_EMPTY (priv->status)) {
 		g_free (priv->status);
 		priv->status = NULL;
+
+		if (error) {
+			DEBUG ("Error getting actual presence message: %s", error->message);
+			g_clear_error (&error);
+		}
 	}
 
 	dbus_g_proxy_connect_signal (DBUS_G_PROXY (priv->mc),
@@ -454,18 +497,9 @@ empathy_idle_init (EmpathyIdle *idle)
 }
 
 EmpathyIdle *
-empathy_idle_new (void)
+empathy_idle_dup_singleton (void)
 {
-	static EmpathyIdle *idle = NULL;
-
-	if (!idle) {
-		idle = g_object_new (EMPATHY_TYPE_IDLE, NULL);
-		g_object_add_weak_pointer (G_OBJECT (idle), (gpointer) &idle);
-	} else {
-		g_object_ref (idle);
-	}
-
-	return idle;
+	return g_object_new (EMPATHY_TYPE_IDLE, NULL);
 }
 
 McPresence
@@ -565,7 +599,7 @@ empathy_idle_set_presence (EmpathyIdle *idle,
 		if (tp_strdiff (priv->status, status)) {
 			g_free (priv->status);
 			priv->status = NULL;
-			if (!G_STR_EMPTY (status)) {
+			if (!EMP_STR_EMPTY (status)) {
 				priv->status = g_strdup (status);
 			}
 			g_object_notify (G_OBJECT (idle), "status");
