@@ -68,12 +68,15 @@ void                     empathy_message_set_body          (EmpathyMessage
 time_t                   empathy_message_get_timestamp     (EmpathyMessage           *message);
 void                     empathy_message_set_timestamp     (EmpathyMessage           *message,
 							    time_t                    timestamp);
-GDate *                  empathy_message_get_date_and_time (EmpathyMessage           *message,
-							    time_t                   *timestamp);
 gboolean                 empathy_message_should_highlight  (EmpathyMessage           *message);
 TpChannelTextMessageType empathy_message_type_from_str     (const gchar              *type_str);
 const gchar *            empathy_message_type_to_str       (TpChannelTextMessageType  type);
 
+guint                    empathy_message_get_id (EmpathyMessage *message);
+void                     empathy_message_set_id (EmpathyMessage *message, guint id);
+
+gboolean                 empathy_message_equal (EmpathyMessage *message1, EmpathyMessage *message2);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_MESSAGE_H__ */
