@@ -32,20 +32,20 @@
 
 #include "empathy-tp-contact-factory.h"
 #include "empathy-utils.h"
+#include "empathy-account-manager.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_TP | EMPATHY_DEBUG_CONTACT
 #include "empathy-debug.h"
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyTpContactFactory)
 typedef struct {
-	MissionControl *mc;
+	EmpathyAccountManager *account_manager;
 	McAccount      *account;
 	TpConnection   *connection;
 	gboolean        ready;
 
 	GList          *contacts;
 	EmpathyContact *user;
-	gpointer        token;
 
 	gchar         **avatar_mime_types;
 	guint           avatar_min_width;
@@ -130,7 +130,7 @@ tp_contact_factory_presences_table_foreach (const gchar    *state_str,
 		message_str = g_value_get_string (message);
 	}
 
-	if (!G_STR_EMPTY (message_str)) {
+	if (!EMP_STR_EMPTY (message_str)) {
 		empathy_contact_set_presence_message (contact, message_str);
 	} else {
 		empathy_contact_set_presence_message (contact, NULL);
@@ -384,7 +384,7 @@ tp_contact_factory_avatar_maybe_update (EmpathyTpContactFactory *tp_factory,
 	}
 
 	/* Check if we have an avatar */
-	if (G_STR_EMPTY (token)) {
+	if (EMP_STR_EMPTY (token)) {
 		empathy_contact_set_avatar (contact, NULL);
 		return TRUE;
 	}
@@ -783,12 +783,12 @@ tp_contact_factory_ready (EmpathyTpContactFactory *tp_factory)
 		handle = empathy_contact_get_handle (contact);
 		id = empathy_contact_get_id (contact);
 		if (handle == 0) {
-			g_assert (!G_STR_EMPTY (id));
+			g_assert (!EMP_STR_EMPTY (id));
 			g_array_append_val (handle_needed, id);
 			handle_needed_contacts = g_list_prepend (handle_needed_contacts,
 								 g_object_ref (contact));
 		}
-		if (G_STR_EMPTY (id)) {
+		if (EMP_STR_EMPTY (id)) {
 			g_array_append_val (id_needed, handle);
 			id_needed_contacts = g_list_prepend (id_needed_contacts,
 							     g_object_ref (contact));
@@ -840,22 +840,19 @@ get_requestable_channel_classes_cb (TpProxy *connection,
 
 	classes = g_value_get_boxed (value);
 	for (i = 0; i < classes->len; i++) {
-		GValue class = {0,};
-		GValue *chan_type, *handle_type;
+		GValueArray *class_struct;
 		GHashTable *fixed_prop;
+		GValue *chan_type, *handle_type;
+		GList *l;
 
-		g_value_init (&class, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
-		g_value_set_static_boxed (&class, g_ptr_array_index (classes, i));
-
-		dbus_g_type_struct_get (&class,
-					0, &fixed_prop,
-					G_MAXUINT);
+		class_struct = g_ptr_array_index (classes, i);
+		fixed_prop = g_value_get_boxed (g_value_array_get_nth (class_struct, 0));
 
 		chan_type = g_hash_table_lookup (fixed_prop,
 			TP_IFACE_CHANNEL ".ChannelType");
 		if (chan_type == NULL ||
 		    tp_strdiff (g_value_get_string (chan_type),
-		    		EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER)) {
+		    		TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER)) {
 			continue;
 		}
 
@@ -869,6 +866,15 @@ get_requestable_channel_classes_cb (TpProxy *connection,
 		/* We can request file transfer channel to contacts. */
 		priv->can_request_ft = TRUE;
 
+		/* Update the capabilities of all contacts */
+		for (l = priv->contacts; l != NULL; l = g_list_next (l)) {
+			EmpathyContact *contact = l->data;
+			EmpathyCapabilities caps;
+
+			caps = empathy_contact_get_capabilities (contact);
+			empathy_contact_set_capabilities (contact, caps |
+				EMPATHY_CAPABILITIES_FT);
+		}
 		break;
 	}
 
@@ -955,13 +961,15 @@ tp_contact_factory_status_updated (EmpathyTpContactFactory *tp_factory)
 {
 	EmpathyTpContactFactoryPriv *priv = GET_PRIV (tp_factory);
 	gboolean                     connection_ready;
+	MissionControl              *mc;
 
 	if (priv->connection) {
 		/* We already have our connection object */
 		return;
 	}
 
-	priv->connection = mission_control_get_tpconnection (priv->mc, priv->account, NULL);
+	mc = empathy_mission_control_dup_singleton ();
+	priv->connection = mission_control_get_tpconnection (mc, priv->account, NULL);
 	if (!priv->connection) {
 		return;
 	}
@@ -979,24 +987,23 @@ tp_contact_factory_status_updated (EmpathyTpContactFactory *tp_factory)
 					  G_CALLBACK (tp_contact_factory_connection_ready_cb),
 					  tp_factory);
 	}
+
+	g_object_unref (mc);
 }
 
 static void
-tp_contact_factory_status_changed_cb (MissionControl           *mc,
-				      TpConnectionStatus        status,
-				      McPresence                presence,
-				      TpConnectionStatusReason  reason,
-				      const gchar              *unique_name,
-				      EmpathyTpContactFactory  *tp_factory)
+tp_contact_factory_account_connection_cb (EmpathyAccountManager *account_manager,
+					  McAccount *account,
+					  TpConnectionStatusReason reason,
+					  TpConnectionStatus current,
+					  TpConnectionStatus previous,
+					  EmpathyTpContactFactory  *tp_factory)
 {
 	EmpathyTpContactFactoryPriv *priv = GET_PRIV (tp_factory);
-	McAccount                   *account;
 
-	account = mc_account_lookup (unique_name);
 	if (account && empathy_account_equal (account, priv->account)) {
 		tp_contact_factory_status_updated (tp_factory);
 	}
-	g_object_unref (account);
 }
 
 static void
@@ -1356,7 +1363,9 @@ tp_contact_factory_finalize (GObject *object)
 	DEBUG ("Finalized: %p (%s)", object,
 		mc_account_get_normalized_name (priv->account));
 
-	empathy_disconnect_account_status_changed (priv->token);
+	g_signal_handlers_disconnect_by_func (priv->account_manager,
+					      tp_contact_factory_account_connection_cb,
+					      object);
 
 	for (l = priv->contacts; l; l = l->next) {
 		g_object_weak_unref (G_OBJECT (l->data),
@@ -1365,7 +1374,7 @@ tp_contact_factory_finalize (GObject *object)
 	}
 
 	g_list_free (priv->contacts);
-	g_object_unref (priv->mc);
+	g_object_unref (priv->account_manager);
 	g_object_unref (priv->account);
 	g_object_unref (priv->user);
 
@@ -1508,12 +1517,13 @@ empathy_tp_contact_factory_init (EmpathyTpContactFactory *tp_factory)
 		EMPATHY_TYPE_TP_CONTACT_FACTORY, EmpathyTpContactFactoryPriv);
 
 	tp_factory->priv = priv;
-	priv->mc = empathy_mission_control_new ();
-	priv->token = empathy_connect_to_account_status_changed (priv->mc,
-						   G_CALLBACK (tp_contact_factory_status_changed_cb),
-						   tp_factory, NULL);
+	priv->account_manager = empathy_account_manager_dup_singleton ();
 
-  priv->can_request_ft = FALSE;
+	g_signal_connect (priv->account_manager, "account-connection-changed",
+			  G_CALLBACK (tp_contact_factory_account_connection_cb),
+			  tp_factory);
+
+	priv->can_request_ft = FALSE;
 }
 
 EmpathyTpContactFactory *
