@@ -59,8 +59,9 @@ typedef struct {
   EmpathyContact *contact;
   /* Tube dispatcher if applicable */
   EmpathyTubeDispatch *tube_dispatch;
-  /* option signal handler */
+  /* option signal handler and it's instance */
   gulong handler;
+  GObject *handler_instance;
   /* optional accept widget */
   GtkWidget *dialog;
 } EventManagerApproval;
@@ -124,6 +125,10 @@ event_manager_approval_free (EventManagerApproval *approval)
     approval->invalidated_handler);
   g_object_unref (approval->operation);
 
+  if (approval->handler != 0)
+    g_signal_handler_disconnect (approval->handler_instance,
+      approval->handler);
+
   if (approval->contact != NULL)
     g_object_unref (approval->contact);
 
@@ -280,7 +285,7 @@ event_text_channel_process_func (EventPriv *event)
     {
       tp_chat = EMPATHY_TP_CHAT
         (empathy_dispatch_operation_get_channel_wrapper (event->approval->operation));
-  
+
       g_signal_handler_disconnect (tp_chat, event->approval->handler);
       event->approval->handler = 0;
     }
@@ -367,8 +372,7 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
-  dialog = gtk_message_dialog_new (GTK_WINDOW (empathy_main_window_get()),
-      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+  dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Incoming call"));
   gtk_message_dialog_format_secondary_text (
     GTK_MESSAGE_DIALOG (dialog),
@@ -622,6 +626,7 @@ event_manager_tube_got_contact_name_cb (EmpathyContact *contact,
         approval->handler = g_signal_connect (approval->tube_dispatch,
           "notify::dispatchability",
           G_CALLBACK (event_manager_tube_dispatch_ability_cb), approval);
+        approval->handler_instance = G_OBJECT (approval->tube_dispatch);
         break;
       case EMPATHY_TUBE_DISPATCHABILITY_POSSIBLE:
         /* fallthrough */
@@ -816,6 +821,7 @@ event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
       /* 1-1 text channel, wait for the first message */
       approval->handler = g_signal_connect (tp_chat, "message-received",
         G_CALLBACK (event_manager_chat_message_received_cb), approval);
+      approval->handler_instance = G_OBJECT (tp_chat);
     }
   else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA))
     {
@@ -838,7 +844,7 @@ event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
         }
 
     }
-  else if (!tp_strdiff (channel_type, EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
     {
       EmpathyContact        *contact;
       gchar                 *header;
