@@ -289,6 +289,7 @@ preferences_sound_cell_toggled_cb (GtkCellRendererToggle *toggle,
 
 	empathy_conf_set_bool (empathy_conf_get (), key, instore);
 
+	g_free (key);
 	gtk_tree_path_free (path);
 }
 
@@ -566,6 +567,7 @@ preferences_languages_load_foreach (GtkTreeModel  *model,
 		}
 	}
 
+	g_free (code);
 	gtk_list_store_set (GTK_LIST_STORE (model), iter, COL_LANG_ENABLED, found, -1);
 	return FALSE;
 }
@@ -688,9 +690,9 @@ preferences_widget_sync_string_combo (const gchar *key, GtkWidget *widget)
 
 	found = FALSE;
 	if (value && gtk_tree_model_get_iter_first (model, &iter)) {
-		gchar *name;
 
 		do {
+			gchar *name;
 			gtk_tree_model_get (model, &iter,
 					    COL_COMBO_NAME, &name,
 					    -1);
@@ -698,13 +700,10 @@ preferences_widget_sync_string_combo (const gchar *key, GtkWidget *widget)
 			if (strcmp (name, value) == 0) {
 				found = TRUE;
 				gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
-				break;
-			} else {
-				found = FALSE;
 			}
 
 			g_free (name);
-		} while (gtk_tree_model_iter_next (model, &iter));
+		} while (!found && gtk_tree_model_iter_next (model, &iter));
 	}
 
 	/* Fallback to the first one. */
